!
!   EMT FORCE & ENERGY CALCULATION
!   for Aluminum
!   P. Stoltze (Jacobsen)
!   calculation of force, total energy, (d(mat) <- for extension)
!   New bookkeep algorithm
!   
!   Feb. 24 (2006)
!   Stomic stress calculation is implemented
!   Oct. 5, 2010
!   A bug in stress calculation for a small cell is fixed

      subroutine emt_St_JCP1990
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: emte0, emte2, emte3 &
       ,emtn0, emteta, emts0, emtgam1, emtbet, emteta2 &
       ,emtrc, emta, emtalp, emtgam2
      double precision :: emtsig1(n),emtsig2(n)
      double precision :: ki,kif
      integer :: istat

      if ((asp(1)/='Al').and.(asp(1)/='al')) then
         print *,'emt_HM_JP1989 is prepared only for aluminum'
         print *, asp(1:2),' is not supported'
         stop
      endif

      istat=1

!   EMT PARAMETER
      ev=1.6021892d-19
      bohr=5.2917706d-11        ! Caution: 0.52 was used in old version
      emte0=-3.28d0*ev
      emte2=1.12d0*ev
      emte3=-0.35d0*ev
      emtn0=0.007d0/(bohr**3)
      emteta=2.0d0/bohr
      emts0=3.0d0*bohr
      emtgam1=1.043d0
      emtbet=1.809d0
      emteta2=1.27d0/bohr
      emtrc=11.50d0*bohr
      emta=-10.0d10
!      emta=-10.0d0
      emtalp=288.36d-49
      emtgam2=1.069d0
!   EMT PARAMETER END

      rc=emtrc*1.1d0
      rc2=rc**2

      emtsig1=0.0d0
      emtsig2=0.0d0

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   First Loop: Chg dns calc

      kns=0
      idbg=0
      do 4200 k=1,n
         kn=knum(k)
         if (kn.eq.0) go to 4200
         do 4300 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(k)-rx(j)
            yy=ry(k)-ry(j)
            zz=rz(k)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  emtthe=1.0d0/(1.0d0+exp(emta*(emtrc-r)))
                  dxx1=exp(-emteta2*r)*emtthe
                  dyy1=exp(-emteta/emtbet*r)*emtthe
                  emtsig1(k)=emtsig1(k)+dxx1
                  emtsig2(k)=emtsig2(k)+dyy1
                  if (j/=k) then
                     emtsig1(j)=emtsig1(j)+dxx1
                     emtsig2(j)=emtsig2(j)+dyy1
                  endif
               endif
            endif
 4300    continue
 4200 continue

!     Calculation of potential energy
      do i=1,n
         emtnbi=emtn0*exp(emteta*emts0)*(emtsig1(i)/12.0d0/emtgam1) &
         **(emteta/emtbet/emteta2)
         emtnfi=emtn0*exp(emteta*emts0)*(emtsig2(i)/12.0d0/emtgam2)
         enpoteach(i)=emte0+emte2*(emtnbi/emtn0-1.0d0)**2 &
         +emte3*(emtnbi/emtn0-1.0d0)**3 &
         +emtalp*(emtnbi-emtnfi)
      enddo

!   Second Loop: Force calc
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      do 4500 i=1,n
         kn=knum(i)
         if (kn.eq.0) go to 4500
         emtnbi=emtn0*exp(emteta*emts0)*(emtsig1(i)/12.0d0/emtgam1) &
          **(emteta/emtbet/emteta2)
         emtnfi=emtn0*exp(emteta*emts0)*(emtsig2(i)/12.0d0/emtgam2)
         dednbi=2.0d0*emte2/emtn0*(emtnbi/emtn0-1.0d0) &
          +3.0d0*emte3/emtn0*(emtnbi/emtn0-1.0d0)**2+emtalp
         dnbds1i=emteta*emtnbi/emtbet/emteta2/emtsig1(i)
         dednfi=-emtalp
         dnfds2i=emtnfi/emtsig2(i)
         do 4600 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  dtdr=emta*exp(emta*(emtrc-r)) &
                  /((1.0d0+exp(emta*(emtrc-r)))**2)
                  emtthe=1.0d0/(1.0d0+exp(emta*(emtrc-r)))
                  ds1dri=exp(-emteta2*r)*(dtdr-emteta2*emtthe)
                  dd1=dednbi*dnbds1i*ds1dri
                  ds2dri=exp(-emteta/emtbet*r) &
                  *(dtdr-emteta/emtbet*emtthe)
                  dd1=dd1+dednfi*dnfds2i*ds2dri

                  emtnbj=emtn0*exp(emteta*emts0) &
                  *(emtsig1(j)/12.0d0/emtgam1) &
                  **(emteta/emtbet/emteta2)

                  dednbj=2.0d0*emte2/emtn0*(emtnbj/emtn0-1.0d0) &
                  +3.0d0*emte3/emtn0*(emtnbj/emtn0-1.0d0)**2+emtalp

                  dnbds1j=emteta*emtnbj/emtbet/emteta2/emtsig1(j)
                  ds1jdr=exp(-emteta2*r)*(dtdr-emteta2*emtthe)
                  dd1=dd1+dednbj*dnbds1j*ds1jdr
                  dednfj=-emtalp
                  emtnfj=emtn0*exp(emteta*emts0) &
                  *(emtsig2(j)/12.0d0/emtgam2)
                  dnfds2j=emtnfj/emtsig2(j)
                  ds2jdr=exp(-emteta/emtbet*r) &
                  *(dtdr-emteta/emtbet*emtthe)
                  dd1=dd1+dednfj*dnfds2j*ds2jdr

!--- Force on j-th atom (and k-th atom) ---
                  f=-dd1

                  ki=f/r
                  if (j==i) ki=ki/2.0d0
                  kif=ki/2.0d0
                  dxx1=ki*xx
                  dyy1=ki*yy
                  dzz1=ki*zz
                  fx(i)=fx(i)+dxx1
                  fy(i)=fy(i)+dyy1
                  fz(i)=fz(i)+dzz1
                  fx(j)=fx(j)-dxx1
                  fy(j)=fy(j)-dyy1
                  fz(j)=fz(j)-dzz1
                  satom(i,1,1)=satom(i,1,1)-kif*xx*xx
                  satom(i,1,2)=satom(i,1,2)-kif*yy*xx
                  satom(i,2,2)=satom(i,2,2)-kif*yy*yy
                  satom(i,1,3)=satom(i,1,3)-kif*zz*xx
                  satom(i,2,3)=satom(i,2,3)-kif*zz*yy
                  satom(i,3,3)=satom(i,3,3)-kif*zz*zz
                  satom(j,1,1)=satom(j,1,1)-kif*xx*xx
                  satom(j,1,2)=satom(j,1,2)-kif*yy*xx
                  satom(j,2,2)=satom(j,2,2)-kif*yy*yy
                  satom(j,1,3)=satom(j,1,3)-kif*zz*xx
                  satom(j,2,3)=satom(j,2,3)-kif*zz*yy
                  satom(j,3,3)=satom(j,3,3)-kif*zz*zz
               endif
            endif
 4600    continue
 4500 continue

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

!      write (10,*) 'EMT force calculation end.'
      end

