!
!   Stillinger-Weber potential
!   for Silicon
!    Frank H. Stillinger and Thomas A. Weber
!    "Computer simulation of local order in condensed phases of silicon"
!    Physical Review B, Vol. 31, No. 8, pp.5262-5271 (1985)
!
!   CALCULATION OF FORCE CONSTANT (Nov. 08)
!

!      module sw_function
!      double precision :: swaa, swbb, swa, swlam, swgam
!      integer :: swp, swq
!      end module

      subroutine sw_forceconst(hessian)
      use sw_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: ki
      integer :: istat
      double precision :: x2(n),y2(n),z2(n),a1(3),a2(3),a3(3),para(7)
      double precision :: hessian(n*3,n*3), inf_dis
      double precision :: rx_s(n),ry_s(n),rz_s(n)
      if (bookkeep_algo /= 2) then
         stop 'Error in tersoff.F: Bookkeep algo is not DOUBLE'
      endif

!--Stillinger-Weber
      pi=4.0d0*atan(1.0d0)
      eps=1.0d-30
      ev2j=1.6021892d-19        !eV -> J
      j2ev=1.0d0/ev2j           !J -> eV
!  PARAMETER 
!  PRB 31 p5262 (1985)
      swaa=7.049556277d0
      swbb=0.6022245584d0
      swp=4
      swq=0
      swa=1.80d0
      swlam=21.0d0
      swgam=1.20d0
      swsig=0.20951d-9
      sweps=3.4723d-19
!--

      istat=1

      rc=swa*swsig
      rc2=rc**2


!   Loop
      inf_dis=0.0005*ang
!      rcc=(rc+inf_dis)*2; rcc2=rcc**2
      rcc=(rc+inf_dis)*2.5; rcc2=rcc**2 !!! range of influence (2 is not enough?)
      hessian=0.0d0
      rx_s(1:n)=rx(1:n); ry_s(1:n)=ry(1:n); rz_s(1:n)=rz(1:n)
      outloop: do ifc=1,6
      inloop: do l=1,n
         rx(1:n)=rx_s(1:n); ry(1:n)=ry_s(1:n); rz(1:n)=rz_s(1:n)
         fx(1:n)=0.0d0; fy(1:n)=0.0d0; fz(1:n)=0.0d0
         if (ifc==1) then; rx(l)=rx(l)+inf_dis; endif
         if (ifc==2) then; rx(l)=rx(l)-inf_dis; endif
         if (ifc==3) then; ry(l)=ry(l)+inf_dis; endif
         if (ifc==4) then; ry(l)=ry(l)-inf_dis; endif
         if (ifc==5) then; rz(l)=rz(l)+inf_dis; endif
         if (ifc==6) then; rz(l)=rz(l)-inf_dis; endif
      kns=0
      iloop: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         disx=abs(rx_s(i)-rx_s(l)); disy=abs(ry_s(i)-ry_s(l)); disz=abs(rz_s(i)-rz_s(l))
         if (disx>rc*3) disx=disx-hmat(1,1)
         if (disy>rc*3) disy=disy-hmat(2,2)
         if (disz>rc*3) disz=disz-hmat(3,3)
         dist2=disx**2+disy**2+disz**2

!         do 200 jj=1,nc
         kns0=kns
         do 200 jcount=1,kn
            kns=kns+1
            if (dist2>rc2*9) go to 201  !!!! skip unless i-l distance < rc*3

            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
            rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
            if (rij2.le.eps) go to 201
            if (rij2.ge.rc2) go to 201
               rij=sqrt(rij2)
               rijs=rij/swsig

               ad1=sweps/swsig*dexp((rijs-swa)**(-1)) &
               *swaa*(-dble(swp)*swbb*rijs**(-swp-1)+dble(swq)*rijs**(-swq-1) &
               +(swbb*rijs**(-swp)-rijs**(-swq))*(-(rijs-swa)**(-2)))
               fx(i)=fx(i)+ad1*(rjx-rix)/rij
               fy(i)=fy(i)+ad1*(rjy-riy)/rij
               fz(i)=fz(i)+ad1*(rjz-riz)/rij

!--------------calc 3-body term---------------------
               knsk=kns0
               do 300 kcount=1,kn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
                  rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2

                  if (rik2.le.eps) go to 301
                  if (rik2.ge.rc2) go to 301
                  rik=sqrt(rik2)
                  riks=rik/swsig
                  rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
                  if (rjk2.le.eps) go to 301
                  rjk=sqrt(rjk2)
                  rjks=rjk/swsig
                  cosjik=(rik2+rij2-rjk2)/(2.0d0*rik*rij)
                  cosijk=(rij2+rjk2-rik2)/(2.0d0*rjk*rij)
                  cosjki=(rjk2+rik2-rij2)/(2.0d0*rjk*rik)

                  dhdr1=-swgam/swsig*(rijs-swa)**(-2)*swh(rijs,riks,cosjik)
                  dhdr2=-swgam/swsig*(riks-swa)**(-2)*swh(rijs,riks,cosjik)

                  dhdc1=dswh(rijs,riks,cosjik)
                  dhdc2=dswh(rijs,rjks,cosijk)
                  dhdc3=dswh(riks,rjks,cosjki)

                  fx(i)=fx(i)+0.5d0*sweps*dhdr1*(rjx-rix)/rij
                  fy(i)=fy(i)+0.5d0*sweps*dhdr1*(rjy-riy)/rij
                  fz(i)=fz(i)+0.5d0*sweps*dhdr1*(rjz-riz)/rij
                  fx(i)=fx(i)+0.5d0*sweps*dhdr2*(rkx-rix)/rik
                  fy(i)=fy(i)+0.5d0*sweps*dhdr2*(rky-riy)/rik
                  fz(i)=fz(i)+0.5d0*sweps*dhdr2*(rkz-riz)/rik
                  fx(i)=fx(i)+0.5d0*sweps*dhdc1/rik2/rij2 &
                  *(cosijk*rik*rjk*(rjx-rix)+cosjki*rij*rjk*(rkx-rix))
                  fy(i)=fy(i)+0.5d0*sweps*dhdc1/rik2/rij2 &
                  *(cosijk*rik*rjk*(rjy-riy)+cosjki*rij*rjk*(rky-riy))
                  fz(i)=fz(i)+0.5d0*sweps*dhdc1/rik2/rij2 &
                  *(cosijk*rik*rjk*(rjz-riz)+cosjki*rij*rjk*(rkz-riz))

                  fx(j)=fx(j)-0.5d0*sweps*dhdr1*(rjx-rix)/rij
                  fy(j)=fy(j)-0.5d0*sweps*dhdr1*(rjy-riy)/rij
                  fz(j)=fz(j)-0.5d0*sweps*dhdr1*(rjz-riz)/rij
                  fx(k)=fx(k)-0.5d0*sweps*dhdr2*(rkx-rix)/rik
                  fy(k)=fy(k)-0.5d0*sweps*dhdr2*(rky-riy)/rik
                  fz(k)=fz(k)-0.5d0*sweps*dhdr2*(rkz-riz)/rik
                  fx(j)=fx(j)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rij/rik*(rkx-rjx)+rjk*cosijk/rij2/rik*(rjx-rix))
                  fy(j)=fy(j)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rij/rik*(rky-rjy)+rjk*cosijk/rij2/rik*(rjy-riy))
                  fz(j)=fz(j)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rij/rik*(rkz-rjz)+rjk*cosijk/rij2/rik*(rjz-riz))
                  fx(k)=fx(k)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rik/rij*(rjx-rkx)+rjk*cosjki/rik2/rij*(rkx-rix))
                  fy(k)=fy(k)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rik/rij*(rjy-rky)+rjk*cosjki/rik2/rij*(rky-riy))
                  fz(k)=fz(k)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rik/rij*(rjz-rkz)+rjk*cosjki/rik2/rij*(rkz-riz))
                  
 301              continue
 300           continue

 201           continue
 200     continue

      end do iloop

      i=l  !!! self term
         if (ifc==1) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)+fz(i)
         else if (ifc==2) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)-fz(i)
         else if (ifc==3) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)+fz(i)
         else if (ifc==4) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)-fz(i)
         else if (ifc==5) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)+fz(i)
         else if (ifc==6) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)-fz(i)
         endif

      kns=0  !!! correlation term
      if (l>1) then
         do i=1,l-1
            kns=kns+knum(i)
         enddo
      endif
      ipre=0

!      do ik=1,knum(l)
!         kns=kns+1
!         i=idpt(kns,1)
      do i=1,n
         if (i==l) cycle  !!! avoid self interaction
         if (i==ipre) cycle  !!! avoid double counting
         ipre=i
         xx=rx_s(l)-rx_s(i); yy=ry_s(l)-ry_s(i); zz=rz_s(l)-rz_s(i)
!!! search replica (using calc_replica leads to error)
         if (xx> rcc) xx=xx-hmat(1,1)
         if (xx<-rcc) xx=xx+hmat(1,1)
         if (yy> rcc) yy=yy-hmat(2,2)
         if (yy<-rcc) yy=yy+hmat(2,2)
         if (zz> rcc) zz=zz-hmat(3,3)
         if (zz<-rcc) zz=zz+hmat(3,3)
         dist2=xx**2+yy**2+zz**2
         if (dist2>rcc2) cycle  !!!! skip unless i-l distance < rcc

         if (ifc==1) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)+fz(i)
         else if (ifc==2) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)-fz(i)
         else if (ifc==3) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)+fz(i)
         else if (ifc==4) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)-fz(i)
         else if (ifc==5) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)+fz(i)
         else if (ifc==6) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)-fz(i)
         endif
      enddo

      end do inloop
      end do outloop

      hessian=-hessian/inf_dis/2.0
      rx(1:n)=rx_s(1:n); ry(1:n)=ry_s(1:n); rz(1:n)=rz_s(1:n)

      end

!      double precision function swh(r1,r2,cost)
!      use sw_function
!      implicit double precision (a-h,o-z)
!      swh=swlam*dexp(swgam*(r1-swa)**(-1)+swgam*(r2-swa)**(-1))*(cost+1.0d0/3.0d0)**2
!      end
!      double precision function dswh(r1,r2,cost)
!      use sw_function
!      implicit double precision (a-h,o-z)
!      dswh=swlam*dexp(swgam*(r1-swa)**(-1)+swgam*(r2-swa)**(-1))*2.0d0*(cost+1.0d0/3.0d0)
!      end

