!
!   EAM FORCE & ENERGY CALCULATION
!   for Silicon
!   Tersoff potential (Ref.**)
!   calculation of force, total energy, d(mat)
!
!   Jan. 13 (2006)
!   For using a modified bookkeep algorithm ('DOUBLE') for 3-body potential
!   Feb. 24 (2006)
!   Stress calculation is implemented (satom(=atomic stress) is also available)

      module tersoff_function_old
      double precision, allocatable :: zmat(:,:)
      double precision :: terrr, terss, teraa, terbb, terlambda, termu, terbeta
      double precision :: tern, terc, terd, terh, terw
      integer :: term
      end module

      subroutine tersoff_old
      use tersoff_function_old
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: ki
      integer :: istat
      if (.not.allocated(zmat)) allocate(zmat(n,n))
      if (bookkeep_algo /= 2) stop 'Error in tersoff.F: Bookkeep algo is not DOUBLE'

!--tersoff
      pi=4.0d0*atan(1.0d0)
      eps=1.0d-20
      ev2j=1.6021892d-19        !eV -> J
      j2ev=1.0d0/ev2j           !J -> eV
      mode=1                    !1:38-14(B,C),2:39-8(Multi),1:Date's type
      mparam=0                  !4:Date's potential(B),5:Yasukawa,6:Date's Potential(read from file)
!CCC   PARAMETER FROM Phys.Rev.B 38-14(1988)p.9902
!C     Si(B) (for surface structure)
      if (mparam.eq.0) then
      terrr=2.80d-10
      terss=3.20d-10
      teraa=3.2647d3*ev2j
      terbb=9.5373d1*ev2j
      terlambda=3.2394d10
      termu=1.3258d10
      terbeta=3.3675d-1
      tern=2.2956d1
      term=3
      terc=4.8381d0
      terd=2.0417d0
      terh=0.0d0
      terw=1.0d0
      endif
!--

      istat=1

      rc=terss
      rc2=terss**2


!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   Loop
      enpot=0.0d0
      enpoteach=0.0d0
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      iloop: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
!         do 200 jj=1,nc
         kns0=kns
         do 200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            zmat(i,j)=0.0d0
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
               rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
               if (rij2.le.eps) go to 201
               rij=sqrt(rij2)
               fij=fcut(rij,terrr,terss)
               dfij=fcutd(rij,terrr,terss)
               if (fij.le.eps) go to 201
               urepij=fij*teraa*exp(-terlambda*rij)
               ad1=teraa*(dfij+fij*(-terlambda))*exp(-terlambda*rij)/rij
!               ad1=(-terlambda)*urepij/rij
               fx(i)=fx(i)+ad1*(rjx-rix)
               fy(i)=fy(i)+ad1*(rjy-riy)
               fz(i)=fz(i)+ad1*(rjz-riz)
!!     Atomic stress
               ad1=ad1*0.5d0
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)+ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)+ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)+ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)+ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)+ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)+ad1*adz*adz
!cc   f'
!               ad1=fcutd(rij,terrr,terss)*teraa*exp(-terlambda*rij)/rij
!               fx(i)=fx(i)+ad1*(rjx-rix)
!               fy(i)=fy(i)+ad1*(rjy-riy)
!               fz(i)=fz(i)+ad1*(rjz-riz)
!--------------calc bij---------------------
               bij=0.0d0
               dzdrx=0.0d0
               dzdry=0.0d0
               dzdrz=0.0d0
               knsk=kns0
               do 300 kcount=1,kn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
                     rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
                     if (rik2.le.eps) go to 301
                     rik=sqrt(rik2)
                     fik=fcut(rik,terrr,terss)
                     if (fik.le.eps) go to 301
                     rjk2=(rkx-rjx)**2 &
                         +(rky-rjy)**2+(rkz-rjz)**2
                     if (rjk2.le.eps) go to 301
                     rjk=sqrt(rjk2)
                     costheta=(rik2+rij2-rjk2)/(2.0d0*rik*rij)
                     if (mode.eq.1) then
                        zetap=fik*exp(termu**term*(rij-rik)**term) &
                            *(1.0d0+terc**2/terd**2 &
                            -terc**2/(terd**2+(terh-costheta)**2))
                     else
                        zetap=fik*terw &
                            *(1.0d0+terc**2/terd**2 &
                            -terc**2/(terd**2+(terh-costheta)**2))
                     endif
                     zmat(i,j)=zmat(i,j)+zetap
                     if (mode.eq.1) then
                        if (term.eq.1) xxx=1.0d0
                        if (term.ne.1) xxx=(rij-rik)**(term-1)
                        dzdrij=termu**term*dble(term) &
!     &                       *(rij-rik)**(term-1) &
                            *xxx &
                            *zetap
                        dzdrik=-dzdrij
                        dzdc=fik*exp(termu**term*(rij-rik)**term) &
                            *(-2.0d0*terc**2*(terh-costheta)) &
                            /((terd**2+(terh-costheta)**2)**2)
                     else
                        dzdrij=0.0d0
                        dzdrik=0.0d0
                        dzdc=fik*terw &
                            *(-2.0d0*terc**2*(terh-costheta)) &
                            /((terd**2+(terh-costheta)**2)**2)
                     endif
! d(cosTheta)/dri
                     ad1=1.0d0/2.0d0/rik2/rij2
                     ad2=rik/rij*(rij2-rik2+rjk2)
                     ad3=rij/rik*(rik2-rij2+rjk2)
!                     dcdrx=ad1*(ad2*(rjx-rix)+ad3*(rkx-rix))
!                     dcdry=ad1*(ad2*(rjy-riy)+ad3*(rky-riy))
!                     dcdrz=ad1*(ad2*(rjz-riz)+ad3*(rkz-riz))
!c                     dcdrx=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rik)
!c     &                    *(rjx-rix)+1.0d0/(rij*rjk)*(rkx-rix)
!c                     dcdry=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rik)
!c     &                    *(rjy-riy)+1.0d0/(rij*rjk)*(rky-riy)
!c                     dcdrz=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rik)
!c     &                    *(rjz-riz)+1.0d0/(rij*rjk)*(rkz-riz)
                     dcdrx=(-rik*(rij2-rik2+rjk2) &
                         /(2.0d0*rik2*rij2*rij))*(rjx-rix) &
                         +(-rij*(rik2-rij2+rjk2) &
                         /(2.0d0*rik2*rik*rij2))*(rkx-rix)
                     dcdry=(-rik*(rij2-rik2+rjk2) &
                         /(2.0d0*rik2*rij2*rij))*(rjy-riy) &
                         +(-rij*(rik2-rij2+rjk2) &
                         /(2.0d0*rik2*rik*rij2))*(rky-riy)
                     dcdrz=(-rik*(rij2-rik2+rjk2) &
                         /(2.0d0*rik2*rij2*rij))*(rjz-riz) &
                         +(-rij*(rik2-rij2+rjk2) &
                         /(2.0d0*rik2*rik*rij2))*(rkz-riz)
!cc   Date's potential ... doubtful
!               dcdrx=1.0d0/rik*((rjx-rix)/rij+(rix-rkx)*costheta/rik) &
!                   -1.0d0/rij*((rix-rkx)/rik+(rjx-rix)*costheta/rij)
!               dcdry=1.0d0/rik*((rjy-riy)/rij+(riy-rky)*costheta/rik) &
!                   -1.0d0/rij*((riy-rky)/rik+(rjy-riy)*costheta/rij)
!               dcdrz=1.0d0/rik*((rjz-riz)/rij+(riz-rkz)*costheta/rik) &
!                   -1.0d0/rij*((riz-rkz)/rik+(rjz-riz)*costheta/rij)
! dZeta/dri
                     dzdrx=dzdrx-(rjx-rix)/rij*dzdrij &
                         -(rkx-rix)/rik*dzdrik &
                         +dcdrx*dzdc
                     dzdry=dzdry-(rjy-riy)/rij*dzdrij &
                         -(rky-riy)/rik*dzdrik &
                         +dcdry*dzdc
                     dzdrz=dzdrz-(rjz-riz)/rij*dzdrij &
                         -(rkz-riz)/rik*dzdrik &
                         +dcdrz*dzdc
 301                 continue
!                  enddo
!                  enddo
!                  enddo
 300           continue
               bij=(1.0d0+(terbeta*zmat(i,j))**tern)**(-0.5d0/tern)
! dbijdr
               if (zmat(i,j).eq.0.0) then
                  ad1=0.0d0
               else
               ad1=-0.5d0*terbeta &
!               ad1=0.5d0*terbeta &
                   *(1.0d0+(terbeta*zmat(i,j))**tern) &
                   **(-0.5d0/tern-1.0d0) &
                   *(terbeta*zmat(i,j))**(tern-1.0d0)
               endif
               dbijdrx=ad1*dzdrx
               dbijdry=ad1*dzdry
               dbijdrz=ad1*dzdrz
               ad1=0.5d0*fij*terbb*exp(-termu*rij)
               fx(i)=fx(i)+ad1*dbijdrx
               fy(i)=fy(i)+ad1*dbijdry
               fz(i)=fz(i)+ad1*dbijdrz
!!     Atomic stress
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)-ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)-ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)-ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)-ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)-ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)-ad1*adz*adz
!--------------calc bij end-----------------
               ushtij=-fij*bij*terbb*exp(-termu*rij)
               ad1=0.5d0*fij*terbb*bij*(termu)*exp(-termu*rij)/rij
               fx(i)=fx(i)+ad1*(rjx-rix)
               fy(i)=fy(i)+ad1*(rjy-riy)
               fz(i)=fz(i)+ad1*(rjz-riz)
!!     Atomic stress
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)+ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)+ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)+ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)+ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)+ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)+ad1*adz*adz
!cc   f'
!               ad1=-fcutd(rij,terrr,terss)*terbb*bij*exp(-termu*rij)/rij*0.5d0
!               fx(i)=fx(i)+ad1*(rjx-rix)
!               fy(i)=fy(i)+ad1*(rjy-riy)
!               fz(i)=fz(i)+ad1*(rjz-riz)
               uij=urepij+ushtij
!               enpot=enpot+uij*0.5d0
               enpoteach(i)=enpoteach(i)+uij*0.5d0
 201           continue
!            enddo
!            enddo
!            enddo
 200     continue
      end do iloop

!      return

!cccc u_ji/r_i
!      irangex=1; irangey=1; irangez=1
!      do 2100 i=1,n
      kns=0
      iloop2: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
!         do 2200 jj=1,nc
         kns0=kns
         do 2200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
               rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
               if (rij2.le.eps) go to 2201
               rij=sqrt(rij2)
               fij=fcut(rij,terrr,terss)
               if (fij.le.eps) go to 2201
!--------------calc bji---------------------
               dzdrx=0.0d0
               dzdry=0.0d0
               dzdrz=0.0d0
               kjn=knum(j)
               knsk=0; if (j>0) then; do kcount=1,j-1; knsk=knsk+knum(kcount); enddo; endif
               do 2300 kcount=1,kjn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(j)-rx(k); yy=ry(j)-ry(k); zz=rz(j)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  rkx=rx(j)-xx; rky=ry(j)-yy; rkz=rz(j)-zz
                     rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
                     if (rik2.le.eps) go to 2301
                     rik=sqrt(rik2)
!                     fik=fcut(rik,terrr,terss)
!                     if (fik.le.eps) go to 2301
                     rjk2=(rkx-rjx)**2 &
                         +(rky-rjy)**2+(rkz-rjz)**2
                     if (rjk2.le.eps) go to 2301
                     rjk=sqrt(rjk2)
                     fjk=fcut(rjk,terrr,terss)
                     if (fjk.le.eps) go to 2301
                     rjk=sqrt(rjk2)
                     costheta=(rjk2+rij2-rik2)/(2.0d0*rjk*rij)

                     if (mode.eq.1) then
                        fjk=fcut(rjk,terrr,terss)
                        zetap=fjk*exp(termu**term*(rij-rjk)**term) &
                            *(1.0d0+terc**2/terd**2 &
                            -terc**2/(terd**2+(terh-costheta)**2))
                        if (term.eq.1) xxx=1.0d0
                        if (term.ne.1) xxx=(rij-rjk)**(term-1)
                        dzdrij=termu**term*dble(term) &
!     &                       *(rij-rjk)**(term-1) &
                            *xxx &
                            *zetap
                        dzdrik=-dzdrij
                        dzdc=fjk*exp(termu**term*(rij-rjk)**term) &
                            *(-2.0d0*terc**2*(terh-costheta)) &
                            /((terd**2+(terh-costheta)**2)**2)
                     else
                        dzdrij=0.0d0
                        dzdrik=0.0d0
                        dzdc=fjk*terw &
                            *(-2.0d0*terc**2*(terh-costheta)) &
                            /((terd**2+(terh-costheta)**2)**2)
                     endif
! d(cosTheta)/dri
                     ad1=1.0d0/2.0d0/rik2/rij2
                     ad2=rik/rij*(rij2-rik2+rjk2)
                     ad3=rij/rik*(rik2-rij2+rjk2)
!                     dcdrx=ad1*(ad2*(rjx-rix)+ad3*(rkx-rix))
!                     dcdry=ad1*(ad2*(rjy-riy)+ad3*(rky-riy))
!                     dcdrz=ad1*(ad2*(rjz-riz)+ad3*(rkz-riz))
! old code
!                     dcdrx=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rik) &
!                         *(rjx-rix)+1.0d0/(rij*rjk)*(rkx-rix)
!                     dcdry=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rik) &
!                         *(rjy-riy)+1.0d0/(rij*rjk)*(rky-riy)
!                     dcdrz=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rik) &
!                         *(rjz-riz)+1.0d0/(rij*rjk)*(rkz-riz)
! mod. 060113
                     dcdrx=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                         *(rjx-rix)+1.0d0/(rij*rjk)*(rkx-rix)
                     dcdry=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                         *(rjy-riy)+1.0d0/(rij*rjk)*(rky-riy)
                     dcdrz=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                         *(rjz-riz)+1.0d0/(rij*rjk)*(rkz-riz)
!cc   Date's program ... doubtful
!                     dcdrx=1.0d0/rij/rjk*(rkx-rjx) &
!                         +costheta/rij2*(rjx-rix)
!                     dcdry=1.0d0/rij/rjk*(rky-rjy) &
!                         +costheta/rij2*(rjy-riy)
!                     dcdrz=1.0d0/rij/rjk*(rkz-rjz) &
!                         +costheta/rij2*(rjz-riz)
! d/dri
!                     dzdrx=dzdrx+(rjx-rix)/rij*dzdrij
!     &                    +(rkx-rix)/rik*dzdrik
!     &                    +dcdrx*dzdc
!                     dzdry=dzdry+(rjy-riy)/rij*dzdrij
!     &                    +(rky-riy)/rik*dzdrik
!     &                    +dcdry*dzdc
!                     dzdrz=dzdrz+(rjz-riz)/rij*dzdrij
!     &                    +(rkz-riz)/rik*dzdrik
!     &                    +dcdrz*dzdc
                     dzdrx=dzdrx-(rjx-rix)/rij*dzdrij &
                         +dcdrx*dzdc
                     dzdry=dzdry-(rjy-riy)/rij*dzdrij &
                         +dcdry*dzdc
                     dzdrz=dzdrz-(rjz-riz)/rij*dzdrij &
                         +dcdrz*dzdc
 2301                continue
 2300          continue
               bji=(1.0d0+(terbeta*zmat(j,i))**tern)**(-0.5d0/tern)
! dbjidr
               if (zmat(j,i).eq.0.0) then
                  ad1=0.0d0
               else
               ad1=-0.5d0*terbeta &
                   *(1.0d0+(terbeta*zmat(j,i))**tern) &
                   **(-0.5d0/tern-1.0d0) &
                   *(terbeta*zmat(j,i))**(tern-1.0d0)
               endif
               dbjidrx=ad1*dzdrx
               dbjidry=ad1*dzdry
               dbjidrz=ad1*dzdrz
               ad1=0.5d0*fij*terbb*exp(-termu*rij)
               fx(i)=fx(i)+ad1*dbjidrx
               fy(i)=fy(i)+ad1*dbjidry
               fz(i)=fz(i)+ad1*dbjidrz
!--------------calc bji end-----------------
               ad1=0.5d0*fij*terbb*bji*(termu)*exp(-termu*rij)/rij
               fx(i)=fx(i)+ad1*(rjx-rix)
               fy(i)=fy(i)+ad1*(rjy-riy)
               fz(i)=fz(i)+ad1*(rjz-riz)
!cc   f'
!               ad1=-fcutd(rij,terrr,terss)*terbb*bji*exp(-termu*rij)/rij*0.5d0
!               fx(i)=fx(i)+ad1*(rjx-rix)
!               fy(i)=fy(i)+ad1*(rjy-riy)
!               fz(i)=fz(i)+ad1*(rjz-riz)
 2201          continue
 2200    continue
! 2100 continue
      end do iloop2

!      return

!cccc u_jk/r_i
!      do 3000 i=1,n
      kns=0
      iloop3: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         do 3100 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
               rji2=(rix-rjx)**2+(riy-rjy)**2+(riz-rjz)**2
               if (rji2.le.eps) go to 3101
               rji=sqrt(rji2)
               fji=fcut(rji,terrr,terss)
               if (fji.le.eps) go to 3101
               kjn=knum(j)
               knsk=0; if (j>0) then; do kcount=1,j-1; knsk=knsk+knum(kcount); enddo; endif
               do 3200 kcount=1,kjn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(j)-rx(k); yy=ry(j)-ry(k); zz=rz(j)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  rkx=rx(j)-xx; rky=ry(j)-yy; rkz=rz(j)-zz
                     rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
                     if (rjk2.le.eps) go to 3201
                     rjk=sqrt(rjk2)
                     fjk=fcut(rjk,terrr,terss)
                     if (fjk.le.eps) go to 3201
                     rki2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
                     if (rki2.le.eps) go to 3201
                     rki=sqrt(rki2)
                     ad1=fjk*terbb*exp(-termu*rjk)
                     ad2=(-1.0d0/2.0d0/tern) &
                         *(1.0d0+(terbeta*zmat(j,k))**tern) &
                         **(-1.0d0/2.0d0/tern-1.0d0)
                     ad3=tern*(terbeta*zmat(j,k))**(tern-1.0d0) &
                         *terbeta
                     dvdz=0.5d0*ad1*ad2*ad3
!     dzdr
                     costheta=(rji2+rjk2-rki2)/(2.0d0*rji*rjk)
                     if (mode.eq.1) then
                        ad4=fji*exp(termu**term*(rjk-rji)**term) &
                            *(-2.0d0*terc**2*(terh-costheta)) &
                            /((terd**2+(terh-costheta)**2)**2)
                     else
                        ad4=fji*terw*(-2.0d0*terc**2*(terh-costheta)) &
                            /((terd**2+(terh-costheta)**2)**2)
                     endif
!                     ad5=(rjk2-rki2-rji2)/2.0d0/rji2/rji/rjk
! old code
!                     ad5=(rjk2-rki2-rji2)/2.0d0/rji2/rji/rki
! mod. 060113
                     ad5=(rjk2-rki2-rji2)/2.0d0/rji2/rji/rjk
                     ad6=1.0d0/rji/rjk
                     dcdrx=(ad5*(rjx-rix)+ad6*(rkx-rix))
                     dcdry=(ad5*(rjy-riy)+ad6*(rky-riy))
                     dcdrz=(ad5*(rjz-riz)+ad6*(rkz-riz))
!cc   Date's program ... doubtful
!                     dcdrx=1.0d0/rji/rjk*(rkx-rjx) &
!                         +costheta/rji2*(rjx-rix)
!                     dcdry=1.0d0/rji/rjk*(rky-rjy) &
!                         +costheta/rji2*(rjy-riy)
!                     dcdrz=1.0d0/rji/rjk*(rkz-rjz) &
!                         +costheta/rji2*(rjz-riz)
!
                     fx(i)=fx(i)+dvdz*ad4*dcdrx
                     fy(i)=fy(i)+dvdz*ad4*dcdry
                     fz(i)=fz(i)+dvdz*ad4*dcdrz
                     if (mode.eq.1) then
                        zetap=fji*exp(termu**term*(rjk-rji)**term) &
                            *(1.0d0+terc**2/terd**2 &
                            -terc**2/(terd**2+(terh-costheta)**2))
                        if (term.eq.1) xxx=1.0d0
                        if (term.ne.1) xxx=(rjk-rji)**(term-1)
                        ad0=-termu**term*dble(term) &
!     &                       *(rjk-rji)**(term-1) &
                            *xxx &
!     &                       *zmat(j,k) &
                            *zetap
                        fx(i)=fx(i)-dvdz*ad0/rji*(rjx-rix)
                        fy(i)=fy(i)-dvdz*ad0/rji*(rjy-riy)
                        fz(i)=fz(i)-dvdz*ad0/rji*(rjz-riz)
                     endif
 3201                continue
 3200          continue
 3101          continue
 3100    continue
      end do iloop3

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

      end

!      double precision function fcut(r,rr,ss)
!      implicit double precision (a-h,o-z)
!      pi=4.0d0*atan(1.0d0)
!      if (r.le.rr) then
!         fcut=1.0d0
!      else if (r.le.ss) then
!         fcut=0.5d0+0.5d0*cos(pi*(r-rr)/(ss-rr))
!      else
!         fcut=0.0d0
!      endif
!      end
!      double precision function fcutd(r,rr,ss)
!      implicit double precision (a-h,o-z)
!      pi=4.0d0*atan(1.0d0)
!      if ((r.lt.ss).and.(r.gt.rr)) then
!         fcutd=-pi/2.0d0*sin(pi*(r-rr)/(ss-rr))/(ss-rr)
!      else
!         fcutd=0.0d0
!      endif
!      end
